@@ -1,31 +1,37 @@
 #!/usr/bin/python3
 
 import gettext
-gettext.install("mintupdate", "/usr/share/linuxmint/locale")
+gettext.install("mintupdate", "/usr/share/locale")
 
 import html
 
+from gi.repository import Gio
+
+settings = Gio.Settings("com.linuxmint.updates")
+if settings.get_boolean("use-lowlatency-kernels"):
+    KERNEL_PKG_NAMES = ['linux-headers-VERSION', 'linux-headers-VERSION-lowlatency', 'linux-image-VERSION-lowlatency', 'linux-modules-VERSION-lowlatency', 'linux-modules-extra-VERSION-lowlatency']
+    KERNEL_PKG_NAMES.append('linux-image-extra-VERSION-lowlatency') # Naming convention in 16.04, until 4.15 series
+else:
+    KERNEL_PKG_NAMES = ['linux-headers-VERSION', 'linux-headers-VERSION-generic', 'linux-image-VERSION-generic', 'linux-modules-VERSION-generic', 'linux-modules-extra-VERSION-generic']
+    KERNEL_PKG_NAMES.append('linux-image-extra-VERSION-generic') # Naming convention in 16.04, until 4.15 series
+
 class Rule():
 
     def __init__(self, name, version, level):
         self.name = name
         self.version = version
         self.level = level
-        self.is_wildcard = False
-        if self.name.startswith("*"):
+        self.is_wildcard = self.name.startswith("*")
+        if self.is_wildcard:
             self.name = self.name.replace("*", "")
-            self.is_wildcard = True
 
     def match(self, pkg_name, pkg_version):
-        matches = False
         if (self.version == "*" or self.version == pkg_version):
             if self.is_wildcard:
-                if (pkg_name.find(self.name) > -1):
-                    matches = True
+                return self.name in pkg_name
             else:
-                if (pkg_name == self.name):
-                    matches = True
-        return matches
+                return pkg_name == self.name
+        return False
 
 class Update():
 
@@ -49,12 +55,14 @@ class Update():
             self.display_name = self.source_name
             self.short_description = package.candidate.raw_description
             self.description = package.candidate.description
+            self.archive = ""
             if (self.new_version != self.old_version):
                 self.type = "package"
                 self.origin = ""
                 for origin in package.candidate.origins:
                     self.origin = origin.origin
                     self.site = origin.site
+                    self.archive = origin.archive
                     if origin.origin == "Ubuntu":
                         self.origin = "ubuntu"
                     elif origin.origin == "Debian":
@@ -74,7 +82,7 @@ class Update():
                         if origin.component == "romeo":
                             self.type = "unstable"
                             break
-                if self.source_name in ["linux", "linux-kernel"]:
+                if package.candidate.section == "kernel" or self.package_name.startswith("linux-headers") or self.real_source_name in ["linux", "linux-kernel", "linux-signed", "linux-meta"]:
                     self.type = "kernel"
 
             self.level = 2 # Level 2 by default
@@ -85,34 +93,37 @@ class Update():
     def add_package(self, pkg):
         self.package_names.append(pkg.name)
         self.size += pkg.candidate.size
-        overwrite_main_package = False
         if self.main_package_name is None or pkg.name == self.source_name:
-            overwrite_main_package = True
-        else:
-            if self.main_package_name == self.source_name:
-                overwrite_main_package = False
-            else:
-                # Overwrite dev, dbg, common, arch packages
-                for suffix in ["-dev", "-dbg", "-common", "-core", "-data", "-doc", ":i386", ":amd64"]:
-                    if (self.main_package_name.endswith(suffix) and not pkg.name.endswith(suffix)):
-                        overwrite_main_package = True
-                        break
-                # Overwrite lib packages
-                for prefix in ["lib", "gir1.2"]:
-                    if (self.main_package_name.startswith(prefix) and not pkg.name.startswith(prefix)):
-                        overwrite_main_package = True
-                        break
-                for keyword in ["-locale-", "-l10n-", "-help-"]:
-                    if (keyword in self.main_package_name) and (keyword not in pkg.name):
-                        overwrite_main_package = True
-                        break
-        if overwrite_main_package:
-            self.description = pkg.candidate.description
-            self.short_description = pkg.candidate.raw_description
-            self.main_package_name  = pkg.name
+            self.overwrite_main_package(pkg)
+            return
+
+        if self.main_package_name != self.source_name:
+            # Overwrite dev, dbg, common, arch packages
+            for suffix in ["-dev", "-dbg", "-common", "-core", "-data", "-doc", ":i386", ":amd64"]:
+                if (self.main_package_name.endswith(suffix) and not pkg.name.endswith(suffix)):
+                    self.overwrite_main_package(pkg)
+                    return
+            # Overwrite lib packages
+            for prefix in ["lib", "gir1.2"]:
+                if (self.main_package_name.startswith(prefix) and not pkg.name.startswith(prefix)):
+                    self.overwrite_main_package(pkg)
+                    return
+            for keyword in ["-locale-", "-l10n-", "-help-"]:
+                if (keyword in self.main_package_name) and (keyword not in pkg.name):
+                    self.overwrite_main_package(pkg)
+                    return
+
+    def overwrite_main_package(self, pkg):
+        self.description = pkg.candidate.description
+        self.short_description = pkg.candidate.raw_description
+        self.main_package_name = pkg.name
 
     def serialize(self):
-        output_string = u"###%d###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s---EOL---" % (self.level, self.display_name, self.source_name, self.real_source_name, self.main_package_name, ", ".join(self.package_names), self.new_version, self.old_version, self.size, self.type, self.origin, self.short_description, self.description, self.site)
+        output_string = u"###%d###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s###%s---EOL---" % \
+        (self.level, self.display_name, self.source_name, self.real_source_name,\
+         self.main_package_name, ", ".join(self.package_names), self.new_version,\
+         self.old_version, self.size, self.type, self.origin, \
+         self.short_description, self.description, self.site, self.archive)
         print(output_string.encode('ascii', 'xmlcharrefreplace'))
 
     def parse(self, input_string):
@@ -121,7 +132,10 @@ class Update():
         except:
             pass
         values = input_string.split("###")
-        nothing, level, self.display_name, self.source_name, self.real_source_name, self.main_package_name, package_names, self.new_version, self.old_version, size, self.type, self.origin, self.short_description, self.description, self.site = values
+        _, level, self.display_name, self.source_name, self.real_source_name,\
+            self.main_package_name, package_names, self.new_version,\
+            self.old_version, size, self.type, self.origin, self.short_description,\
+            self.description, self.site, self.archive = values
         self.level = int(level)
         self.size = int(size)
         for package_name in package_names.split(", "):
