@@ -4,7 +4,6 @@ import apt
 import codecs
 import fnmatch
 import gettext
-import gi
 import os
 import platform
 import re
@@ -14,9 +13,9 @@ import traceback
 
 from gi.repository import Gio
 
-from Classes import Update, Alias, Rule
+from Classes import Update, Alias, Rule, KERNEL_PKG_NAMES
 
-gettext.install("mintupdate", "/usr/share/linuxmint/locale")
+gettext.install("mintupdate", "/usr/share/locale")
 
 class KernelVersion():
 
@@ -40,9 +39,6 @@ class KernelVersion():
 # nothing else is listed.
 PRIORITY_UPDATES = ['mintupdate', 'mint-upgrade-info']
 
-KERNEL_PKG_NAMES = ['linux-headers-VERSION', 'linux-headers-VERSION-generic', 'linux-image-VERSION-generic', 'linux-modules-VERSION-generic', 'linux-modules-extra-VERSION-generic']
-KERNEL_PKG_NAMES.append('linux-image-extra-VERSION-generic') # Naming convention in 16.04, until 4.15 series
-
 class APTCheck():
 
     def __init__(self):
@@ -85,17 +81,12 @@ class APTCheck():
 
     def refresh_cache(self):
         if os.getuid() == 0 :
-            use_synaptic = False
-            if (len(sys.argv) > 1):
-                if sys.argv[1] == "--use-synaptic":
-                    use_synaptic = True
-
-            if use_synaptic:
+            if "--use-synaptic" in sys.argv:
                 window_id = int(sys.argv[2])
-                from subprocess import Popen, PIPE
+                from subprocess import Popen
                 cmd = ["sudo", "/usr/sbin/synaptic", "--hide-main-window", "--update-at-startup", "--non-interactive", "--parent-window-id", "%d" % window_id]
                 comnd = Popen(' '.join(cmd), shell=True)
-                returnCode = comnd.wait()
+                comnd.wait()
             else:
                 self.cache.update()
 
@@ -113,42 +104,54 @@ class APTCheck():
                 self.add_update(pkg)
 
         # Kernel updates
+        if self.settings.get_boolean("use-lowlatency-kernels"):
+            kernel_type = "-lowlatency"
+        else:
+            kernel_type = "-generic"
+
+        meta_names = []
+        _metas = [s for s in self.cache.keys() if s.startswith("linux" + kernel_type)]
+        for meta in _metas:
+            shortname = meta.split(":")[0]
+            if shortname not in meta_names:
+                meta_names.append(shortname)
+
         try:
             if self.settings.get_boolean("kernel-updates-are-visible"):
+
                 # Get the uname version
                 uname_kernel = KernelVersion(platform.release())
 
-                # Get the recommended version
-                if 'linux-image-generic' in self.cache:
-                    recommended_kernel = KernelVersion(self.cache['linux-image-generic'].candidate.version)
-                    if (uname_kernel.numeric_representation <= recommended_kernel.numeric_representation):
-                        for pkgname in KERNEL_PKG_NAMES:
-                            pkgname = pkgname.replace('VERSION', recommended_kernel.std_version)
-                            if pkgname in self.cache:
-                                pkg = self.cache[pkgname]
-                                if not pkg.is_installed:
-                                    self.add_update(pkg, kernel_update=True)
-                        return
+                # Check if any meta is installed..
+                meta_installed = False
+                for meta_name in meta_names:
+                    if meta_name in self.cache:
+                        meta = self.cache[meta_name]
+                        if meta.is_installed:
+                            meta_installed = True
+                            return
 
-                # We're using a series which is more recent than the recommended one, check the HWE kernel first
-                if 'linux-image-generic-hwe-16.04' in self.cache:
-                    recommended_kernel = KernelVersion(self.cache['linux-image-generic-hwe-16.04'].candidate.version)
-                    if (uname_kernel.numeric_representation <= recommended_kernel.numeric_representation):
-                        for pkgname in KERNEL_PKG_NAMES:
-                            pkgname = pkgname.replace('VERSION', recommended_kernel.std_version)
-                            if pkgname in self.cache:
-                                pkg = self.cache[pkgname]
-                                if not pkg.is_installed:
-                                    self.add_update(pkg, kernel_update=True)
-                        return
+                # If no meta is installed, try to recommend one
+                if not meta_installed:
+                    for meta_name in meta_names:
+                        if meta_name in self.cache:
+                            meta = self.cache[meta_name]
+                            recommended_kernel = KernelVersion(meta.candidate.version)
+                            if (uname_kernel.numeric_representation <= recommended_kernel.numeric_representation):
+                                # This meta version is >= to the uname version, add it as an update
+                                self.add_update(meta, kernel_update=True)
+                                # Return because we only want to add one meta, and we don't want to recommend latest kernels in the series
+                                return
 
                 # We've gone past all the metas, so we should recommend the latest kernel on the series we're in
                 max_kernel = uname_kernel
-                for pkg in self.cache:
-                    package_name = pkg.name
-                    if (package_name.startswith("linux-image-3") or package_name.startswith("linux-image-4")) and package_name.endswith("-generic"):
-                        version = package_name.replace("linux-image-", "").replace("-generic", "")
-                        kernel = KernelVersion(version)
+                kernel_type = "-generic"
+                if self.settings.get_boolean("use-lowlatency-kernels"):
+                    kernel_type = "-lowlatency"
+                for pkgname in self.cache.keys():
+                    match = re.match(r'^(?:linux-image-)(?:unsigned-)?(\d.+?)' + kernel_type + '$', pkgname)
+                    if match:
+                        kernel = KernelVersion(match.group(1))
                         if kernel.numeric_representation > max_kernel.numeric_representation and kernel.series == max_kernel.series:
                             max_kernel = kernel
                 if max_kernel.numeric_representation != uname_kernel.numeric_representation:
@@ -159,7 +162,7 @@ class APTCheck():
                             if not pkg.is_installed:
                                 self.add_update(pkg, kernel_update=True)
 
-        except Exception as e:
+        except Exception:
             print(sys.exc_info()[0])
 
     def add_update(self, package, kernel_update=False):
@@ -173,7 +176,7 @@ class APTCheck():
         else:
             source_name = package.candidate.source_name
 
-        # ignore blacklisted packages
+        # ignore packages blacklisted by the user
         for blacklist in self.settings.get_strv("blacklisted-packages"):
             if fnmatch.fnmatch(source_name, blacklist):
                 return
@@ -200,7 +203,8 @@ class APTCheck():
                             update.level = rule.level
                             break
 
-                self.updates[source_name] = update
+                if update.level < 5:
+                    self.updates[source_name] = update
             if kernel_update:
                 update.type = "kernel"
 
@@ -297,7 +301,7 @@ class APTCheck():
                                                         newline = True
                                                     else:
                                                         if (newline):
-                                                            value = "%s%s" % (value, dline.capitalize())
+                                                            value = "%s%s" % (value, self.capitalize(dline))
                                                         else:
                                                             value = "%s %s" % (value, dline)
                                                         newline = False
@@ -331,12 +335,18 @@ class APTCheck():
                 update.short_description = update.short_description.split("\n")[0]
             if update.short_description.endswith("."):
                 update.short_description = update.short_description[:-1]
-            update.short_description = update.short_description.capitalize()
+            update.short_description = self.capitalize(update.short_description)
             if "& " in update.short_description:
                 update.short_description = update.short_description.replace('&', '&amp;')
             if "& " in update.description:
                 update.description = update.description.replace('&', '&amp;')
 
+    def capitalize(self, string):
+        if len(string) > 1:
+            return (string[0].upper() + string[1:])
+        else:
+            return (string)
+
 if __name__ == "__main__":
     try:
         check = APTCheck()
@@ -347,9 +357,16 @@ if __name__ == "__main__":
         check.apply_aliases()
         check.clean_descriptions()
         check.serialize_updates()
+        if os.path.exists("/usr/bin/mintinstall-update-pkgcache"):
+            # Spawn the cache update asynchronously
+            # We're using os.system with & here to make sure it's async and detached
+            # from the caller (which will die before the child process is finished)
+            # stdout/stderr is also directed to /dev/null so it doesn't interfere
+            # or block the output from checkAPT
+            os.system("/usr/bin/mintinstall-update-pkgcache > /dev/null 2>&1 &")
     except Exception as error:
         print("CHECK_APT_ERROR---EOL---")
         print(sys.exc_info()[0])
-        print(error)
+        print("Error: %s" % error)
         traceback.print_exc()
         sys.exit(1)
