@@ -10,7 +10,7 @@ from subprocess import Popen, PIPE
 import os, apt, time
 import configparser
 
-gettext.install("mintupdate", "/usr/share/linuxmint/locale")
+gettext.install("mintupdate", "/usr/share/locale")
 
 class Assistant:
 
@@ -234,7 +234,7 @@ class Assistant:
             else:
                 os.system("gsettings set %s false" % screensaver_setting)
 
-        cmd = ["/usr/bin/mint-release-upgrade-root", "%s" % self.current_codename, "%s" % self.assistant.get_window().get_xid()]
+        cmd = ["pkexec", "/usr/bin/mint-release-upgrade-root", "%s" % self.current_codename, "%s" % self.assistant.get_window().get_xid()]
         comnd = Popen(' '.join(cmd), shell=True)
         returnCode = comnd.wait()
 
@@ -252,23 +252,20 @@ class Assistant:
                         break
 
         if new_codename == self.rel_target_codename:
-            vbox_content = Gtk.HBox()
-            image = Gtk.Image.new_from_file('/usr/lib/linuxmint/mintUpdate/rel_upgrades/success.png')
-            vbox_content.pack_start(image, False, False, 0)
-            label = Gtk.Label()
-            label.set_line_wrap(True)
-            label.set_markup(_("Your operating system was successfully upgraded. Please reboot your computer for all changes to take effect."))
-            vbox_content.pack_start(label, False, False, 6)
-            self.vbox_summary.pack_start(vbox_content, False, False, 6)
+            image_result = "success"
+            message_text = _("Your operating system was successfully upgraded. Please reboot your computer for all changes to take effect.")
         else:
-            vbox_content = Gtk.HBox()
-            image = Gtk.Image.new_from_file('/usr/lib/linuxmint/mintUpdate/rel_upgrades/failure.png')
-            vbox_content.pack_start(image, False, False, 0)
-            label = Gtk.Label()
-            label.set_line_wrap(True)
-            label.set_markup(_("The upgrade did not succeed. Make sure you are connected to the Internet and try to upgrade again."))
-            vbox_content.pack_start(label, False, False, 6)
-            self.vbox_summary.pack_start(vbox_content, False, False, 6)
+            image_result = "failure"
+            message_text = _("The upgrade did not succeed. Make sure you are connected to the Internet and try to upgrade again.")
+
+        vbox_content = Gtk.HBox()
+        image = Gtk.Image.new_from_file('/usr/lib/linuxmint/mintUpdate/rel_upgrades/%s.png' % image_result)
+        vbox_content.pack_start(image, False, False, 0)
+        label = Gtk.Label()
+        label.set_line_wrap(True)
+        label.set_markup(message_text)
+        vbox_content.pack_start(label, False, False, 6)
+        self.vbox_summary.pack_start(vbox_content, False, False, 6)
 
         self.vbox_summary.show_all()
         self.assistant.set_page_complete(self.vbox_summary, True)
